"use client";

// src/option.tsx
import { mergeRefs as mergeRefs2 } from "@webbo-ui/react-utils";
import { usePointerEvents } from "@webbo-ui/use-pointer-events";
import React2 from "react";

// src/autocomplete.tsx
import * as Popper from "@webbo-ui/popper";
import { useControllableState } from "@webbo-ui/use-controllable-state";
import { CustomError } from "@webbo-ui/error";
import lodashGroupBy from "lodash.groupby";
import {
  autocomplete
} from "@webbo-ui/theme";
import { mergeRefs } from "@webbo-ui/react-utils";
import React from "react";
import { createPortal } from "react-dom";
import { createContextScope } from "@webbo-ui/context";
import { jsx, jsxs } from "react/jsx-runtime";
import { createElement } from "react";
var Autocomplete_Name = "Autocomplete";
var [StylesProvider, stylesHook] = createContextScope(Autocomplete_Name);
var useStylesContext = stylesHook;
var AutocompleteImp = React.forwardRef((props, ref) => {
  const {
    classNames,
    className,
    offset,
    isOpen: openProp,
    onOpenChange,
    defaultOpen = false,
    defaultValue,
    value: valueProp,
    onChange,
    shadow = "md",
    options: optionsProp = [],
    disabled,
    multiple,
    disableClearable,
    disableCloseOnSelect,
    getOptionDisabled,
    noOptionsText = "no options",
    loading,
    loadingText = "loading ...",
    getOptionLabel: getOptionLabelProp,
    getOptionKey,
    groupBy,
    inputValue: inputValueProp,
    onInputChange: onInputChangeProp,
    filterOptions,
    renderInput,
    disablePortal,
    disablePopper,
    renderOption,
    ...restProps
  } = props;
  const getOptionLabel = (option) => {
    if (loading)
      return "";
    if (getOptionLabelProp) {
      return getOptionLabelProp(option);
    }
    if (!("label" in option))
      throw new CustomError(
        "Select",
        "consider to add `label` property in all options or use `getOptionLabel` prop to get option label"
      );
    return option.label;
  };
  const [value, setValue] = useControllableState({
    defaultValue,
    value: valueProp,
    onChange: (value2, reason) => {
      if (!reason)
        throw new CustomError(
          "Autocomplete",
          "internal Error, reason is not defined"
        );
      onChange?.({ target: { value: value2 } }, reason, value2);
    }
  });
  const inputDefaultValue = !Array.isArray(value) && value ? getOptionLabel(value) : "";
  const prevSelectedValue = React.useRef(inputDefaultValue);
  const [inputValue, setInputValue] = useControllableState({
    defaultValue: inputDefaultValue,
    value: inputValueProp,
    onChange: onInputChangeProp
  });
  const [options, setOptions] = React.useState(optionsProp);
  const groupedOptions = React.useMemo(() => {
    if (!groupBy)
      return null;
    const grouped = lodashGroupBy(options, (opt) => {
      const by = groupBy(opt);
      if (!isNaN(+by))
        return "0-9";
      return by;
    });
    return Object.entries(grouped).sort((a, b) => {
      const a_key = a[0];
      const b_key = b[0];
      if (a_key < b_key)
        return -1;
      if (a_key > b_key)
        return 1;
      return 0;
    }).reduce(
      (acc, ele) => (acc[ele[0]] = ele[1], acc),
      {}
    );
  }, [groupBy, options]);
  const inputRef = React.useRef(null);
  const popperReferenceRef = React.useRef(null);
  const [focused, setFocused] = React.useState(null);
  const lisboxId = React.useId();
  const [isOpen, setIsOpen] = useControllableState({
    defaultValue: defaultOpen ?? false,
    value: openProp,
    onChange: onOpenChange
  });
  const handleClose = () => {
    setIsOpen(false);
    setFocused(null);
    setOptions(optionsProp);
    if (Array.isArray(value))
      setInputValue("");
    if (prevSelectedValue.current !== inputValue)
      setInputValue(prevSelectedValue.current);
  };
  const onOpen = () => {
    if (isOpen)
      return;
    setIsOpen(true);
    if (!options.length)
      return;
    if (!value) {
      setFocused(getNext(options[0], options, getOptionDisabled));
      return;
    }
    setFocused(Array.isArray(value) ? value[0] : value);
  };
  const onSelect = (option) => {
    if (Array.isArray(value)) {
      const val = value.find((ele) => ele === option) ? value.filter((ele) => ele !== option) : [...value, option];
      setValue(val, "select");
      setFocused(option);
      setInputValue("");
    }
    if (!Array.isArray(value)) {
      setValue(option, "select");
      setFocused(option);
      const val = getOptionLabel(option);
      setInputValue(val);
      prevSelectedValue.current = val;
    }
    if (!disableCloseOnSelect)
      handleClose();
    setOptions(optionsProp);
  };
  const onHover = (option) => {
    setFocused(option);
  };
  const handleKeyDown = (e) => {
    const ArrowDown = e.key === "ArrowDown";
    const ArrowUp = e.key === "ArrowUp";
    const Escape = e.key === "Escape";
    const Enter = e.key === "Enter";
    const Home = e.key === "Home";
    const End = e.key === "End";
    if (ArrowDown || ArrowUp)
      e.preventDefault();
    if (ArrowDown && !isOpen) {
      onOpen();
      return;
    }
    if (!options.length || !isOpen)
      return;
    if (Escape) {
      handleClose();
      return;
    }
    if (Enter && focused) {
      e.preventDefault();
      onSelect(focused);
      return;
    }
    if ((ArrowDown || ArrowUp) && !focused) {
      setFocused(getNext(options[0], options, getOptionDisabled));
      return;
    }
    if (ArrowDown && focused && focused !== options[options.length - 1]) {
      setFocused(getNext(options[options.indexOf(focused) + 1], options));
      return;
    }
    if (ArrowUp && focused && focused !== options[0]) {
      setFocused(getPrevious(options[options.indexOf(focused) - 1], options));
      return;
    }
    if (Home) {
      setFocused(getNext(options[0], options));
      return;
    }
    if (End) {
      setFocused(getPrevious(options[options.length - 1], options));
      return;
    }
  };
  const onClear = () => {
    inputRef.current?.focus();
    setFocused(null);
    setInputValue("");
    prevSelectedValue.current = "";
    setOptions(optionsProp);
    if (multiple) {
      setValue([], "clear");
    } else {
      setValue(null, "clear");
    }
  };
  const handleInputChange = (e) => {
    if (loading)
      return;
    const val = e.target.value;
    setInputValue(val);
    setIsOpen(true);
    if (!val) {
      setOptions(optionsProp);
      setFocused(null);
      return;
    }
    const filter = filterOptions || ((options2, inputValue2) => options2.filter(
      (opt) => getOptionLabel(opt).toLowerCase().startsWith(inputValue2.toLowerCase())
    ));
    setOptions(filter(optionsProp, val));
  };
  const getOptionId = (ele) => getOptionKey?.(ele) ?? getOptionLabel(ele).replaceAll(" ", "-");
  const styles = React.useMemo(() => autocomplete({ shadow }), [shadow]);
  const getOptionProps = (ele) => ({
    key: getOptionId(ele),
    option: ele,
    onHover: () => onHover(ele),
    onSelect: () => onSelect(ele),
    id: getOptionId(ele),
    className: styles.option({ className: classNames?.option }),
    state: {
      focused: ele === focused,
      selected: Array.isArray(value) ? !!value.find((val) => val === ele) : ele === value,
      disabled: getOptionDisabled?.(ele) ?? false
    }
  });
  if (multiple && !Array.isArray(value))
    throw new CustomError(
      "Autocomplete",
      "value must be an Array when multiple is true"
    );
  if (!multiple && Array.isArray(value))
    throw new CustomError(
      "Autocomplete",
      "value must not be an Array when multiple is false"
    );
  if (!renderInput)
    throw new CustomError("Autocomplete", "`renderInput` prop is required");
  const listBox = /* @__PURE__ */ jsxs(
    "ul",
    {
      ...restProps,
      ref,
      id: lisboxId,
      className: styles.listbox({
        className: classNames?.listbox ?? className
      }),
      role: "listbox",
      "aria-multiselectable": multiple,
      "aria-roledescription": multiple ? "multiple select list" : "single select list",
      onPointerDown: (e) => e.preventDefault(),
      children: [
        !loading && options.length && !groupBy ? options.map((option) => {
          const label = getOptionLabel(option);
          const props2 = getOptionProps(option);
          return /* @__PURE__ */ createElement(Option, { ...props2, key: props2.key }, renderOption?.({
            option,
            label,
            state: props2.state
          }) ?? label);
        }) : null,
        !loading && options.length && groupBy && groupedOptions ? Object.entries(groupedOptions).map(([groupHeader, grouped]) => /* @__PURE__ */ jsxs(
          "li",
          {
            className: styles.group({
              className: classNames?.group
            }),
            children: [
              /* @__PURE__ */ jsx(
                "div",
                {
                  className: styles.groupHeader({
                    className: classNames?.groupHeader
                  }),
                  children: groupHeader
                }
              ),
              /* @__PURE__ */ jsx(
                "ul",
                {
                  className: styles.groupItems({
                    className: classNames?.groupItems
                  }),
                  children: grouped.map((option) => {
                    const label = getOptionLabel(option);
                    const props2 = getOptionProps(option);
                    return /* @__PURE__ */ createElement(Option, { ...props2, key: props2.key }, renderOption?.({
                      option,
                      label,
                      state: props2.state
                    }) ?? label);
                  })
                }
              )
            ]
          },
          groupHeader.replaceAll(" ", "-")
        )) : null,
        !loading && !options?.length ? /* @__PURE__ */ jsx(
          "div",
          {
            className: styles.noOptions({
              className: classNames?.noOptions
            }),
            children: noOptionsText
          }
        ) : null,
        loading ? /* @__PURE__ */ jsx(
          "div",
          {
            className: styles.loading({
              className: classNames?.noOptions
            }),
            children: loadingText
          }
        ) : null
      ]
    }
  );
  const withPopper = disablePopper ? listBox : /* @__PURE__ */ jsx(Popper.Floating, { sticky: "always", mainOffset: offset || 5, children: listBox });
  return /* @__PURE__ */ jsxs(Popper.Root, { children: [
    /* @__PURE__ */ jsx(Popper.Reference, { children: ({ referenceRef }) => renderInput({
      selected: Array.isArray(value) ? value.map((opt) => ({
        label: getOptionLabel(opt),
        onDelete: () => {
          setValue(
            (prev) => Array.isArray(prev) ? prev.filter((ele) => ele !== opt) : null,
            "remove"
          );
        }
      })) : null,
      onBlur: handleClose,
      isOpen,
      multiple: !!multiple,
      onOpen,
      clearButtonProps: {
        onClear,
        onPointerDown: (e) => e.preventDefault(),
        tabIndex: -1,
        "aria-label": "clear value button"
      },
      showClearButton: disableClearable ? false : Array.isArray(value) ? !!value.length : !!value,
      popperReferenceRef: mergeRefs(referenceRef, popperReferenceRef),
      inputRef,
      inputValue,
      onChange: handleInputChange,
      disabled: !!disabled,
      onKeyDown: handleKeyDown,
      loading: !!loading,
      ariaProps: {
        role: "combobox",
        "aria-expanded": isOpen,
        "aria-controls": lisboxId,
        "aria-haspopup": "listbox",
        "aria-autocomplete": "list",
        "aria-activedescendant": isOpen && focused ? getOptionId(focused) : void 0
      }
    }) }),
    /* @__PURE__ */ jsx(StylesProvider, { ...styles, children: isOpen ? disablePortal ? withPopper : createPortal(withPopper, document.body) : null })
  ] });
});
AutocompleteImp.displayName = "webbo-ui." + Autocomplete_Name;
var Autocomplete = AutocompleteImp;
var getNext = (current, items, isOptionDisabled) => {
  if (!isOptionDisabled)
    return current;
  for (let i = items.indexOf(current); i <= items.length; i++) {
    const option = items[i];
    if (!isOptionDisabled(option))
      return option;
  }
  return current;
};
var getPrevious = (current, items, isOptionDisabled) => {
  if (!isOptionDisabled)
    return current;
  for (let i = items.indexOf(current) - 1; i > 0; i--) {
    const option = items[i];
    if (!isOptionDisabled(option))
      return option;
  }
  return current;
};

// src/option.tsx
import { jsx as jsx2 } from "react/jsx-runtime";
var Option_Name = "Option";
var Option = React2.forwardRef(
  (props, ref) => {
    const {
      onSelect,
      onHover,
      state: { disabled, focused, selected },
      children,
      id,
      className
    } = props;
    const styles = useStylesContext(Option_Name);
    const innerRef = React2.useRef(null);
    const isHovered = React2.useRef(false);
    const pointerEvents = usePointerEvents({
      onPress: () => {
        if (disabled)
          return;
        onSelect();
      }
    });
    React2.useEffect(() => {
      if (selected)
        innerRef.current?.scrollIntoView({
          behavior: "instant",
          block: "center"
        });
    }, [selected]);
    React2.useEffect(() => {
      if (focused && !isHovered.current)
        innerRef.current?.scrollIntoView({
          behavior: "instant",
          block: "nearest"
        });
    }, [focused]);
    return /* @__PURE__ */ jsx2(
      "li",
      {
        ref: mergeRefs2(ref, innerRef),
        onPointerEnter: () => {
          isHovered.current = true;
          onHover();
        },
        onPointerLeave: () => {
          isHovered.current = false;
        },
        className: styles.option({ className }),
        id,
        role: "option",
        "aria-selected": disabled ? void 0 : selected,
        "data-disabled": disabled,
        "data-selected": selected,
        "data-focused": focused,
        ...pointerEvents,
        children
      }
    );
  }
);
Option.displayName = "webbo-ui." + Option_Name;

export {
  Option,
  useStylesContext,
  Autocomplete
};
